@@ -16,7 +16,7 @@ class Exponential(nn.Module):
         return torch.exp(input)
 
 
-class BaselineSequential(nn.Module):
+class FlexibleSequential(nn.Module):
     def __init__(
         self,
         n_in,
@@ -24,9 +24,11 @@ class BaselineSequential(nn.Module):
         layers,
         hidden_dim,
         Activator=nn.ReLU,
-        LastLayer=nn.Identity,
         hidden_bias=True,
+        LastActivator=nn.Identity,
         last_bias=True,
+        RescalingLayer=None,
+        rescaling_layer_kwargs=[]
     ):
         """
         Init method.
@@ -37,9 +39,16 @@ class BaselineSequential(nn.Module):
         self.layers = layers
         self.hidden_dim = hidden_dim
         self.Activator = Activator
-        self.LastLayer = LastLayer
+        self.LastActivator = LastActivator
         self.hidden_bias = hidden_bias
         self.last_bias = last_bias
+        self.rescaling_layer_args = rescaling_layer_kwargs
+        self.RescalingLayer = RescalingLayer # must map n_in to (n_out x n_out) matrix
+
+        if not self.RescalingLayer == None:
+            self.rescale = RescalingLayer(**rescaling_layer_kwargs)  # construct as required
+        else:
+            self.rescale = None
 
         # Constructor
         self.model = nn.Sequential(
@@ -48,21 +57,23 @@ class BaselineSequential(nn.Module):
             # Add in layers - 1
             *[
                 nn.Sequential(
-                    nn.Linear(
-                        self.hidden_dim,
-                        self.hidden_dim,
-                        bias=self.hidden_bias,
-                    ),
+                    nn.Linear(self.hidden_dim, self.hidden_dim, bias=self.hidden_bias),
                     self.Activator(),
                 )
                 for i in range(self.layers - 1)
             ],
             nn.Linear(self.hidden_dim, self.n_out, bias=self.last_bias),
-            self.LastLayer()
+            self.LastActivator()
         )
 
     def forward(self, input):
-        return self.model(input)  # pass through to the stored net
+        out = self.model(input)  # pass through to the stored net
+        if not self.RescalingLayer is None:
+            # The rescaling should take n_in -> a n_out x n_out matrix
+            # then out = model(input)*rescale(input) 
+            return out # TODO
+        else:
+            return out
 
     def string(self):
         return self.model.string()  # dispay as
